@@ -1040,19 +1040,39 @@ class _Window:
             )
         )
 
+        # Remove any windows that aren't in the server's stack
+        windows = list(filter(lambda w: w[0].wid in stack, windows))
+
         # Sort this list to match stacking order reported by server
         windows.sort(key=lambda w: stack.index(w[0].wid))
 
         # Get lists of windows on lower, higher or same "layer" as window
-        lower = [w[0].wid for w in windows if w[1] is None and w[2] > layering]
-        higher = [w[0].wid for w in windows if w[1] is None and w[2] < layering]
-        same = [w[0].wid for w in windows if w[1] is None and w[2] == layering]
+        lower = [w[0].wid for w in windows if w[2] > layering]
+        higher = [w[0].wid for w in windows if w[2] < layering]
+        same = [w[0].wid for w in windows if w[2] == layering]
 
         # We now need to identify the new position in the stack
 
         # If the window has a parent, the window should just be put above it
-        if parent:
-            sibling = parent
+        # If the parent isn't being managed by qtile then it may not be stacked correctly
+        if parent and parent in self.qtile.windows_map:
+            # If the window is modal then it should be placed above every other window that is in that window group
+            # e.g. the parent of the dialog and any other window that is also transient for the same parent.
+            if "_NET_WM_STATE_MODAL" in self.window.get_net_wm_state():
+                window_group = [parent]
+                window_group.extend(
+                    k
+                    for k, v in self.qtile.windows_map.items()
+                    if v.window.get_wm_transient_for() == parent
+                )
+                window_group.sort(key=stack.index)
+
+                # Make sure we're above the last window in that group
+                sibling = window_group[-1]
+
+            else:
+                sibling = parent
+
             above = True
 
         # Now we just check whether the window has changed layer.
@@ -1166,9 +1186,31 @@ class _Window:
             stackmode=xcffib.xproto.StackMode.Above if above else xcffib.xproto.StackMode.Below,
             sibling=sibling,
         )
-        # TODO: also move our children if we were moved upwards
+
+        # Move window's children if we were moved upwards
+        if above:
+            self.raise_children(stack=stack)
+
         self.qtile.core.update_client_lists()
 
+    def raise_children(self, stack=None):
+        """Ensure any transient windows are moved up with the parent."""
+        children = [
+            k
+            for k, v in self.qtile.windows_map.items()
+            if v.window.get_wm_transient_for() == self.window.wid
+        ]
+        if children:
+            if stack is None:
+                stack = list(self.qtile.core._root.query_tree())
+            parent = self.window.wid
+            children.sort(key=stack.index)
+            for child in children:
+                self.qtile.windows_map[child].window.configure(
+                    stackmode=xcffib.xproto.StackMode.Above, sibling=parent
+                )
+                parent = child
+
     def paint_borders(self, color, width):
         self.borderwidth = width
         self.bordercolor = color
@@ -1288,12 +1330,12 @@ class _Window:
         self.qtile.core._root.set_property("_NET_ACTIVE_WINDOW", self.window.wid)
         self._ungrab_click()
 
-        if self.group:
-            self.group.current_window = self
-
         # Check if we need to restack a previously focused fullscreen window
         self.qtile.core.check_stacking(self)
 
+        if self.group and self.group.current_window is not self:
+            self.group.focus(self)
+
         hook.fire("client_focus", self)
 
     @expose_command()
@@ -1622,6 +1664,8 @@ class Static(_Window, base.Static):
     def bring_to_front(self):
         if self.get_wm_type() != "desktop":
             self.window.configure(stackmode=xcffib.xproto.StackMode.Above)
+            self.raise_children()
+            self.qtile.core.update_client_lists()
 
 
 class Window(_Window, base.Window):
@@ -2243,6 +2287,8 @@ class Window(_Window, base.Window):
     def bring_to_front(self):
         if self.get_wm_type() != "desktop":
             self.window.configure(stackmode=xcffib.xproto.StackMode.Above)
+            self.raise_children()
+            self.qtile.core.update_client_lists()
 
     def _is_in_window(self, x, y, window):
         return window.edges[0] <= x <= window.edges[2] and window.edges[1] <= y <= window.edges[3]
