@@ -37,6 +37,7 @@
 
 from __future__ import annotations
 
+import contextlib
 import functools
 import operator
 from itertools import chain, repeat
@@ -62,15 +63,6 @@ class XCBQError(QtileError):
     pass
 
 
-def rdict(d):
-    r = {}
-    for k, v in d.items():
-        r.setdefault(v, []).append(k)
-    return r
-
-
-rkeysyms = rdict(keysyms)
-
 # Keyboard modifiers bitmask values from X Protocol
 ModMasks = {
     "shift": 1 << 0,
@@ -86,6 +78,7 @@ ModMasks = {
 AllButtonsMask = 0b11111 << 8
 ButtonMotionMask = 1 << 13
 ButtonReleaseMask = 1 << 3
+PointerMotionHintMask = 1 << 7
 
 NormalHintsFlags = {
     "USPosition": 1,  # User-specified x, y
@@ -221,6 +214,13 @@ XCB_CONN_ERRORS = {
     7: "XCB_CONN_CLOSED_FDPASSING_FAILED",
 }
 
+# Some opcodes from xproto.h, used for faking input.
+XCB_KEY_PRESS = 2
+XCB_KEY_RELEASE = 3
+XCB_BUTTON_PRESS = 4
+XCB_BUTTON_RELEASE = 5
+XCB_MOTION_NOTIFY = 6
+
 
 class MaskMap:
     """
@@ -336,8 +336,9 @@ class Screen(_Wrapper):
             return desired_depth, self._visuals[desired_depth]
 
         logger.info(
-            f"{desired_depth} bit colour depth not available. "
-            f"Falling back to root depth: {self.root_depth}."
+            "%s bit colour depth not available. Falling back to root depth: %s.",
+            desired_depth,
+            self.root_depth,
         )
         return self.root_depth, self._visuals[self.root_depth]
 
@@ -351,7 +352,7 @@ class Screen(_Wrapper):
         """
         allowed = screen.allowed_depths
         if depth not in [x.depth for x in allowed]:
-            logger.warning(f"Unsupported colour depth: {depth}.")
+            logger.warning("Unsupported colour depth: %s", depth)
             return
 
         for i in allowed:
@@ -390,7 +391,12 @@ class Colormap:
             def x8to16(i):
                 return 0xFFFF * (i & 0xFF) // 0xFF
 
-            color = hex(color)
+            try:
+                color = hex(color)
+            except ValueError:
+                logger.error("Colormap failed to allocate %s", color)
+                color = "#ff0000"
+
             r = x8to16(int(color[-6] + color[-5], 16))
             g = x8to16(int(color[-4] + color[-3], 16))
             b = x8to16(int(color[-2] + color[-1], 16))
@@ -442,42 +448,6 @@ class XFixes:
         self.conn.xfixes.ext.SelectSelectionInput(window.wid, _selection, self.selection_mask)
 
 
-class NetWmState:
-    """NetWmState is a descriptor for _NET_WM_STATE_* properties"""
-
-    def __init__(self, prop_name):
-        self.prop_name = prop_name
-
-    def __get__(self, xcbq_win, cls):
-        try:
-            atom = self.atom
-        except AttributeError:
-            atom = xcbq_win.conn.atoms[self.prop_name]
-            self.atom = atom
-        reply = xcbq_win.get_property("_NET_WM_STATE", "ATOM", unpack=int)
-        if atom in reply:
-            return True
-        return False
-
-    def __set__(self, xcbq_win, value):
-        try:
-            atom = self.atom
-        except AttributeError:
-            atom = xcbq_win.conn.atoms[self.prop_name]
-            self.atom = atom
-
-        value = bool(value)
-        reply = list(xcbq_win.get_property("_NET_WM_STATE", "ATOM", unpack=int))
-        is_set = atom in reply
-        if is_set and not value:
-            reply.remove(atom)
-            xcbq_win.set_property("_NET_WM_STATE", reply)
-        elif value and not is_set:
-            reply.append(atom)
-            xcbq_win.set_property("_NET_WM_STATE", reply)
-        return
-
-
 class Connection:
     _extmap = {
         "xinerama": Xinerama,
@@ -634,10 +604,8 @@ class Connection:
         return window.XWindow(self, wid)
 
     def disconnect(self):
-        try:
+        with contextlib.suppress(xcffib.ConnectionException):
             self.conn.disconnect()
-        except xcffib.ConnectionException:
-            logger.error("Failed to disconnect, connection already failed?")
         self._connected = False
 
     def flush(self):
@@ -693,8 +661,8 @@ class Painter:
         try:
             with open(image_path, "rb") as f:
                 image, _ = cairocffi.pixbuf.decode_to_image_surface(f.read())
-        except IOError as e:
-            logger.error("Wallpaper: %s" % e)
+        except IOError:
+            logger.exception("Could not load wallpaper:")
             return
 
         # Querying the screen dimensions via the xcffib connection does not
@@ -705,9 +673,13 @@ class Painter:
         width = max((win.x + win.width for win in root_windows))
         height = max((win.y + win.height for win in root_windows))
 
-        root_pixmap = self.default_screen.root.get_property(
-            "_XROOTPMAP_ID", xcffib.xproto.Atom.PIXMAP, int
-        )
+        try:
+            root_pixmap = self.default_screen.root.get_property(
+                "_XROOTPMAP_ID", xcffib.xproto.Atom.PIXMAP, int
+            )
+        except xcffib.ConnectionException:
+            root_pixmap = None
+
         if not root_pixmap:
             root_pixmap = self.default_screen.root.get_property(
                 "ESETROOT_PMAP_ID", xcffib.xproto.Atom.PIXMAP, int
@@ -788,7 +760,7 @@ class Painter:
 
 
 def get_keysym(key: str) -> int:
-    keysym = keysyms.get(key)
+    keysym = keysyms.get(key.lower())
     if not keysym:
         raise XCBQError("Unknown key: %s" % key)
     return keysym
@@ -810,7 +782,7 @@ def translate_masks(modifiers: list[str]) -> int:
     masks = []
     for i in modifiers:
         try:
-            masks.append(ModMasks[i])
+            masks.append(ModMasks[i.lower()])
         except KeyError as e:
             raise XCBQError("Unknown modifier: %s" % i) from e
     if masks:
