@@ -30,11 +30,12 @@ from libqtile.layout.floating import Floating
 from libqtile.layout.matrix import Matrix
 from libqtile.layout.max import Max
 from libqtile.layout.ratiotile import RatioTile
+from libqtile.layout.screensplit import ScreenSplit
 from libqtile.layout.slice import Slice
 from libqtile.layout.spiral import Spiral
 from libqtile.layout.stack import Stack
 from libqtile.layout.tile import Tile
 from libqtile.layout.tree import TreeTab
 from libqtile.layout.verticaltile import VerticalTile
-from libqtile.layout.xmonad import MonadTall, MonadWide, MonadThreeCol
+from libqtile.layout.xmonad import MonadTall, MonadThreeCol, MonadWide
 from libqtile.layout.zoomy import Zoomy
