@@ -26,11 +26,20 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
 
 import math
+from typing import TYPE_CHECKING
 
+from libqtile.command.base import expose_command
 from libqtile.layout.base import _SimpleLayoutBase
 
+if TYPE_CHECKING:
+    from typing import Any, Self
+
+    from libqtile.backend.base import Window
+    from libqtile.group import _Group
+
 ROWCOL = 1  # do rows at a time left to right top down
 COLROW = 2  # do cols top to bottom, left to right
 
@@ -71,7 +80,6 @@ class GridInfo:
         best_ratio = None
         best_rows_cols_orientation = None
         for rows, cols, orientation in self._possible_grids(num_windows):
-
             sample_width = width / cols
             sample_height = height / rows
             sample_ratio = sample_width / sample_height
@@ -218,14 +226,14 @@ class RatioTile(_SimpleLayoutBase):
         self.last_size = None
         self.last_screen = None
 
-    def clone(self, group):
+    def clone(self, group: _Group) -> Self:
         return _SimpleLayoutBase.clone(self, group)
 
-    def add(self, w):
+    def add_client(self, w: Window) -> None:  # type: ignore[override]
         self.dirty = True
         self.clients.append_head(w)
 
-    def remove(self, w):
+    def remove(self, w: Window) -> Window | None:
         self.dirty = True
         return _SimpleLayoutBase.remove(self, w)
 
@@ -269,7 +277,8 @@ class RatioTile(_SimpleLayoutBase):
         )
         win.unhide()
 
-    def info(self):
+    @expose_command()
+    def info(self) -> dict[str, Any]:
         d = _SimpleLayoutBase.info(self)
         focused = self.clients.current_client
         d["ratio"] = self.ratio
@@ -277,29 +286,35 @@ class RatioTile(_SimpleLayoutBase):
         d["layout_info"] = self.layout_info
         return d
 
-    cmd_down = _SimpleLayoutBase.previous
-    cmd_up = _SimpleLayoutBase.next
+    @expose_command("down")
+    def previous(self) -> None:
+        _SimpleLayoutBase.previous(self)
 
-    cmd_previous = _SimpleLayoutBase.previous
-    cmd_next = _SimpleLayoutBase.next
+    @expose_command("up")
+    def next(self) -> None:
+        _SimpleLayoutBase.next(self)
 
-    def cmd_shuffle_down(self):
+    @expose_command()
+    def shuffle_down(self):
         if self.clients:
             self.clients.rotate_up()
             self.group.layout_all()
 
-    def cmd_shuffle_up(self):
+    @expose_command()
+    def shuffle_up(self):
         if self.clients:
             self.clients.rotate_down()
             self.group.layout_all()
 
-    def cmd_decrease_ratio(self):
+    @expose_command()
+    def decrease_ratio(self):
         new_ratio = self.ratio - self.ratio_increment
         if new_ratio < 0:
             return
         self.ratio = new_ratio
         self.group.layout_all()
 
-    def cmd_increase_ratio(self):
+    @expose_command()
+    def increase_ratio(self):
         self.ratio += self.ratio_increment
         self.group.layout_all()
