@@ -28,10 +28,21 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from libqtile import hook
+from libqtile.command.base import expose_command
+from libqtile.config import ScreenRect
 from libqtile.layout.base import Layout
 
+if TYPE_CHECKING:
+    from typing import Any, Self, Sequence
+
+    from libqtile.backend import base
+    from libqtile.group import _Group
+
 to_superscript = dict(zip(map(ord, "0123456789"), map(ord, "⁰¹²³⁴⁵⁶⁷⁸⁹")))
 
 
@@ -43,7 +54,7 @@ class TreeNode:
         self._children_top = None
         self._children_bot = None
 
-    def add(self, node, hint=None):
+    def add_client(self, node, hint=None):
         """Add a node below this node
 
         The `hint` is a node to place the new node after in this nodes
@@ -156,7 +167,7 @@ class Root(TreeNode):
         else:
             self.def_section = self.sections[default_section]
 
-    def add(self, win, hint=None):
+    def add_client(self, win, hint=None):
         """Add a new window
 
         Adds a new `Window` to the tree.  The location of the new node is
@@ -181,7 +192,7 @@ class Root(TreeNode):
             parent = self.def_section
 
         node = Window(win)
-        parent.add(node, hint=hint)
+        parent.add_client(node, hint=hint)
         return node
 
     def add_section(self, name):
@@ -278,7 +289,7 @@ class Window(TreeNode):
             return self
         return super().button_press(x, y)
 
-    def remove(self):
+    def remove(self) -> None:
         """Removes this Window
 
         If this window has children, the first child takes the place of this
@@ -287,10 +298,10 @@ class Window(TreeNode):
         if self.children:
             head = self.children[0]
             # add the first child to our parent, next to ourselves
-            self.parent.add(head, hint=self)
+            self.parent.add_client(head, hint=self)
             # move remaining children to be under the new head
             for i in self.children[1:]:
-                head.add(i)
+                head.add_client(i)
 
         self.parent.children.remove(self)
         del self.children
@@ -369,7 +380,7 @@ class TreeTab(Layout):
         ("section_padding", 4, "Bottom of margin section label"),
         ("section_left", 4, "Left margin of section label"),
         ("panel_width", 150, "Width of the left panel"),
-        ("sections", ["Default"], "Foreground color of inactive tab"),
+        ("sections", ["Default"], "Titles of section instances"),
         ("previous_on_rm", False, "Focus previous window on close instead of first."),
         ("place_right", False, "Place the tab panel on the right side"),
     ]
@@ -384,7 +395,7 @@ class TreeTab(Layout):
         self._tree = Root(self.sections)
         self._nodes = {}
 
-    def clone(self, group):
+    def clone(self, group: _Group) -> Self:
         c = Layout.clone(self, group)
         c._focused = None
         c._panel = None
@@ -394,40 +405,44 @@ class TreeTab(Layout):
     def focus(self, win):
         self._focused = win
 
-    def focus_first(self):
+    def focus_first(self) -> base.Window | None:
         win = self._tree.get_first_window()
         if win:
             return win.window
+        return None
 
-    def focus_last(self):
+    def focus_last(self) -> base.Window | None:
         win = self._tree.get_last_window()
         if win:
             return win.window
+        return None
 
-    def focus_next(self, client):
+    def focus_next(self, client: base.Window) -> base.Window | None:
         win = self._nodes[client].get_next_window()
         if win:
             return win.window
+        return None
 
-    def focus_previous(self, client):
+    def focus_previous(self, client: base.Window) -> base.Window | None:
         win = self._nodes[client].get_prev_window()
         if win:
             return win.window
+        return None
 
-    def blur(self):
+    def blur(self) -> None:
         # Does not clear current window, will change if new one
         # will be focused. This works better when floating window
         # will be next focused one
         pass
 
-    def add(self, win):
+    def add_client(self, win):
         if self._focused:
-            node = self._tree.add(win, hint=self._nodes[self._focused])
+            node = self._tree.add_client(win, hint=self._nodes[self._focused])
         else:
-            node = self._tree.add(win)
+            node = self._tree.add_client(win)
         self._nodes[win] = node
 
-    def remove(self, win):
+    def remove(self, win: base.Window) -> None:
         if win not in self._nodes:
             return
 
@@ -465,7 +480,7 @@ class TreeTab(Layout):
         if node:
             self.group.focus(node.window, False)
 
-    def configure(self, client, screen_rect):
+    def configure(self, client: base.Window, screen_rect: ScreenRect) -> None:
         if self._nodes and client is self._focused:
             client.place(
                 screen_rect.x, screen_rect.y, screen_rect.width, screen_rect.height, 0, None
@@ -474,7 +489,7 @@ class TreeTab(Layout):
         else:
             client.hide()
 
-    def finalize(self):
+    def finalize(self) -> None:
         if self._panel:
             self._panel.kill()
         Layout.finalize(self)
@@ -488,7 +503,8 @@ class TreeTab(Layout):
                 clients.append(window.window)
         return clients
 
-    def info(self):
+    @expose_command()
+    def info(self) -> dict[str, Any]:
         def show_section_tree(root):
             """
             Show a section tree in a nested list, whose every element has the form:
@@ -533,7 +549,7 @@ class TreeTab(Layout):
         d["client_trees"] = trees
         return d
 
-    def show(self, screen_rect):
+    def show(self, screen_rect: ScreenRect) -> None:
         if not self._panel:
             self._create_panel(screen_rect)
         if self.place_right:
@@ -543,11 +559,12 @@ class TreeTab(Layout):
         self._resize_panel(panel)
         self._panel.unhide()
 
-    def hide(self):
+    def hide(self) -> None:
         if self._panel:
             self._panel.hide()
 
-    def cmd_down(self):
+    @expose_command("down")
+    def next(self) -> None:
         """Switch down in the window list"""
         win = None
         if self._focused:
@@ -558,9 +575,8 @@ class TreeTab(Layout):
             self.group.focus(win.window, False)
         self._focused = win.window if win else None
 
-    cmd_next = cmd_down
-
-    def cmd_up(self):
+    @expose_command("up")
+    def previous(self) -> None:
         """Switch up in the window list"""
         win = None
         if self._focused:
@@ -571,9 +587,8 @@ class TreeTab(Layout):
             self.group.focus(win.window, False)
         self._focused = win.window if win else None
 
-    cmd_previous = cmd_up
-
-    def cmd_move_up(self):
+    @expose_command()
+    def move_up(self):
         win = self._focused
         if not win:
             return
@@ -585,7 +600,8 @@ class TreeTab(Layout):
             p[idx - 1] = node
         self.draw_panel()
 
-    def cmd_move_down(self):
+    @expose_command()
+    def move_down(self):
         win = self._focused
         if not win:
             return
@@ -597,27 +613,31 @@ class TreeTab(Layout):
             p[idx + 1] = node
         self.draw_panel()
 
-    def cmd_move_left(self):
+    @expose_command()
+    def move_left(self):
         win = self._focused
         if not win:
             return
         node = self._nodes[win]
         if not isinstance(node.parent, Section):
             node.parent.children.remove(node)
-            node.parent.parent.add(node)
+            node.parent.parent.add_client(node)
         self.draw_panel()
 
-    def cmd_add_section(self, name):
+    @expose_command()
+    def add_section(self, name):
         """Add named section to tree"""
         self._tree.add_section(name)
         self.draw_panel()
 
-    def cmd_del_section(self, name):
+    @expose_command()
+    def del_section(self, name):
         """Remove named section from tree"""
         self._tree.del_section(name)
         self.draw_panel()
 
-    def cmd_section_up(self):
+    @expose_command()
+    def section_up(self):
         win = self._focused
         if not win:
             return
@@ -628,10 +648,11 @@ class TreeTab(Layout):
         idx = snode.parent.children.index(snode)
         if idx > 0:
             node.parent.children.remove(node)
-            snode.parent.children[idx - 1].add(node)
+            snode.parent.children[idx - 1].add_client(node)
         self.draw_panel()
 
-    def cmd_section_down(self):
+    @expose_command()
+    def section_down(self):
         win = self._focused
         if not win:
             return
@@ -642,10 +663,11 @@ class TreeTab(Layout):
         idx = snode.parent.children.index(snode)
         if idx < len(snode.parent.children) - 1:
             node.parent.children.remove(node)
-            snode.parent.children[idx + 1].add(node)
+            snode.parent.children[idx + 1].add_client(node)
         self.draw_panel()
 
-    def cmd_sort_windows(self, sorter, create_sections=True):
+    @expose_command()
+    def sort_windows(self, sorter, create_sections=True):
         """Sorts window to sections using sorter function
 
         Parameters
@@ -674,7 +696,8 @@ class TreeTab(Layout):
                 win.parent = nsec
         self.draw_panel()
 
-    def cmd_move_right(self):
+    @expose_command()
+    def move_right(self):
         win = self._focused
         if not win:
             return
@@ -682,30 +705,40 @@ class TreeTab(Layout):
         idx = node.parent.children.index(node)
         if idx > 0:
             node.parent.children.remove(node)
-            node.parent.children[idx - 1].add(node)
+            node.parent.children[idx - 1].add_client(node)
         self.draw_panel()
 
-    def cmd_expand_branch(self):
+    @expose_command()
+    def expand_branch(self):
         if not self._focused:
             return
         self._nodes[self._focused].expanded = True
         self.draw_panel()
 
-    def cmd_collapse_branch(self):
+    @expose_command()
+    def collapse_branch(self):
         if not self._focused:
             return
         self._nodes[self._focused].expanded = False
         self.draw_panel()
 
-    def cmd_increase_ratio(self):
+    @expose_command()
+    def increase_ratio(self):
         self.panel_width += 10
         self.group.layout_all()
 
-    def cmd_decrease_ratio(self):
+    @expose_command()
+    def decrease_ratio(self):
         self.panel_width -= 10
         self.group.layout_all()
 
     def _create_drawer(self, screen_rect):
+        # Create a new drawer object if the screen is a different height
+        # e.g. if moving between screens with different resolutions
+        if self._drawer is not None and self._drawer.height != screen_rect.height:
+            self._drawer.finalize()
+            self._drawer = None
+
         if self._drawer is None:
             self._drawer = self._panel.create_drawer(
                 self.panel_width,
@@ -716,7 +749,7 @@ class TreeTab(Layout):
             "", "ffffff", self.font, self.fontsize, self.fontshadow, wrap=False
         )
 
-    def layout(self, windows, screen_rect):
+    def layout(self, windows: Sequence[base.Window], screen_rect: ScreenRect) -> None:
         if self.place_right:
             body, panel = screen_rect.hsplit(screen_rect.width - self.panel_width)
         else:
