@@ -35,7 +35,7 @@ import os
 import platform
 import re
 import warnings
-from abc import ABC, abstractclassmethod
+from abc import ABC, abstractmethod
 from enum import Enum, unique
 from pathlib import Path
 from subprocess import CalledProcessError, check_output
@@ -59,7 +59,8 @@ class BatteryState(Enum):
     DISCHARGING = 2
     FULL = 3
     EMPTY = 4
-    UNKNOWN = 5
+    NOT_CHARGING = 5
+    UNKNOWN = 6
 
 
 BatteryStatus = NamedTuple(
@@ -79,7 +80,7 @@ class _Battery(ABC):
     battery implementation should provide.
     """
 
-    @abstractclassmethod
+    @abstractmethod
     def update_status(self) -> BatteryStatus:
         """Read the battery status
 
@@ -107,7 +108,7 @@ def load_battery(**config) -> _Battery:
     """
     system = platform.system()
     if system == "FreeBSD":
-        return _FreeBSDBattery(str(config["battery"]))
+        return _FreeBSDBattery(str(config.get("battery", 0)))
     elif system == "Linux":
         return _LinuxBattery(**config)
     else:
@@ -192,7 +193,7 @@ class _LinuxBattery(_Battery, configurable.Configurable):
         ),
     ]
 
-    filenames = {}  # type: dict
+    filenames: dict = {}
 
     BAT_DIR = "/sys/class/power_supply"
 
@@ -236,7 +237,7 @@ class _LinuxBattery(_Battery, configurable.Configurable):
             with open(path, "r") as f:
                 return f.read().strip(), value_type
         except OSError as e:
-            logger.debug("Failed to read '{}': {}".format(path, e))
+            logger.debug("Failed to read '%s':", path, exc_info=True)
             if isinstance(e, FileNotFoundError):
                 # Let's try another file if this one doesn't exist
                 return None
@@ -275,6 +276,8 @@ class _LinuxBattery(_Battery, configurable.Configurable):
             state = BatteryState.CHARGING
         elif stat == "Discharging":
             state = BatteryState.DISCHARGING
+        elif stat == "Not charging":
+            state = BatteryState.NOT_CHARGING
         else:
             state = BatteryState.UNKNOWN
 
@@ -320,6 +323,7 @@ class Battery(base.ThreadPoolText):
         ("discharge_char", "V", "Character to indicate the battery is discharging"),
         ("full_char", "=", "Character to indicate the battery is full"),
         ("empty_char", "x", "Character to indicate the battery is empty"),
+        ("not_charging_char", "*", "Character to indicate the batter is not charging"),
         ("unknown_char", "?", "Character to indicate the battery status is unknown"),
         ("format", "{char} {percent:2.0%} {hour:d}:{min:02d} {watt:.2f} W", "Display format"),
         ("hide_threshold", None, "Hide the text when there is enough energy 0 <= x < 1"),
@@ -430,6 +434,8 @@ class Battery(base.ThreadPoolText):
             if self.show_short_text:
                 return "Empty"
             char = self.empty_char
+        elif status.state == BatteryState.NOT_CHARGING:
+            char = self.not_charging_char
         else:
             char = self.unknown_char
 
@@ -451,12 +457,12 @@ class BatteryIcon(base._Widget):
     """Battery life indicator widget."""
 
     orientations = base.ORIENTATION_HORIZONTAL
-    defaults = [
+    defaults: list[tuple[str, Any, str]] = [
         ("battery", 0, "Which battery should be monitored"),
         ("update_interval", 60, "Seconds between status updates"),
         ("theme_path", default_icon_path(), "Path of the icons"),
         ("scale", 1, "Scale factor relative to the bar height.  " "Defaults to 1"),
-    ]  # type: list[tuple[str, Any, str]]
+    ]
 
     icon_names = (
         "battery-missing",
@@ -481,12 +487,12 @@ class BatteryIcon(base._Widget):
 
         base._Widget.__init__(self, length=bar.CALCULATED, **config)
         self.add_defaults(self.defaults)
-        self.scale = 1.0 / self.scale  # type: float
+        self.scale: float = 1.0 / self.scale
 
         self.length_type = bar.STATIC
         self.length = 0
         self.image_padding = 0
-        self.surfaces = {}  # type: dict[str, Img]
+        self.images: dict[str, Img] = {}
         self.current_icon = "battery-missing"
 
         self._battery = self._load_battery(**config)
@@ -518,7 +524,7 @@ class BatteryIcon(base._Widget):
             img.resize(height=new_height)
             if img.width > self.length:
                 self.length = int(img.width + self.image_padding * 2)
-            self.surfaces[key] = img.pattern
+            self.images[key] = img
 
     def update(self) -> None:
         status = self._battery.update_status()
@@ -529,8 +535,12 @@ class BatteryIcon(base._Widget):
 
     def draw(self) -> None:
         self.drawer.clear(self.background or self.bar.background)
-        self.drawer.ctx.set_source(self.surfaces[self.current_icon])
+        image = self.images[self.current_icon]
+        self.drawer.ctx.save()
+        self.drawer.ctx.translate(0, (self.bar.height - image.height) // 2)
+        self.drawer.ctx.set_source(image.pattern)
         self.drawer.ctx.paint()
+        self.drawer.ctx.restore()
         self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.length)
 
     @staticmethod
