@@ -24,29 +24,39 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
 import mailbox
 import os.path
-from typing import Dict
 
 from libqtile.widget import base
 
 
 class Maildir(base.ThreadPoolText):
     """A simple widget showing the number of new mails in maildir mailboxes"""
-    orientations = base.ORIENTATION_HORIZONTAL
+
     defaults = [
         ("maildir_path", "~/Mail", "path to the Maildir folder"),
-        ("sub_folders", [], 'The subfolders to scan (e.g. [{"path": "INBOX", '
-            '"label": "Home mail"}, {"path": "spam", "label": "Home junk"}]'),
+        (
+            "sub_folders",
+            [{"path": "INBOX", "label": "Home mail"}, {"path": "spam", "label": "Home junk"}],
+            "List of subfolders to scan. Each subfolder is a dict of `path` " "and `label`.",
+        ),
         ("separator", " ", "the string to put between the subfolder strings."),
-        ("total", False, "Whether or not to sum subfolders into a grand \
-            total. The first label will be used."),
-        ("hide_when_empty", False, "Whether not to display anything if "
-         "the subfolder has no new mail"),
+        (
+            "total",
+            False,
+            "Whether or not to sum subfolders into a grand \
+            total. The first label will be used.",
+        ),
+        (
+            "hide_when_empty",
+            False,
+            "Whether not to display anything if " "the subfolder has no new mail",
+        ),
         ("empty_color", None, "Display color when no new mail is available"),
         ("nonempty_color", None, "Display color when new mail is available"),
-        ("subfolder_fmt", "{label}: {value}",
-         "Display format for one subfolder"),
+        ("subfolder_fmt", "{label}: {value}", "Display format for one subfolder"),
     ]
 
     def __init__(self, **config):
@@ -56,10 +66,7 @@ class Maildir(base.ThreadPoolText):
         # if it looks like a list of strings then we just convert them
         # and use the name as the label
         if isinstance(self.sub_folders[0], str):
-            self.sub_folders = [
-                {"path": folder, "label": folder}
-                for folder in self.sub_folders
-            ]
+            self.sub_folders = [{"path": folder, "label": folder} for folder in self.sub_folders]
 
     def poll(self):
         """Scans the mailbox for new messages
@@ -75,8 +82,7 @@ class Maildir(base.ThreadPoolText):
                 yield path.rsplit(":")[0]
 
         for sub_folder in self.sub_folders:
-            path = os.path.join(os.path.expanduser(self.maildir_path),
-                                sub_folder["path"])
+            path = os.path.join(os.path.expanduser(self.maildir_path), sub_folder["path"])
             maildir = mailbox.Maildir(path)
             state[sub_folder["label"]] = 0
 
@@ -96,15 +102,14 @@ class Maildir(base.ThreadPoolText):
         if color is None:  # default to self.foreground
             return s
 
-        return s.join(('<span foreground="{}">'.format(color),
-                       '</span>'))
+        return s.join(('<span foreground="{}">'.format(color), "</span>"))
 
-    def format_text(self, state: Dict[str, int]) -> str:
+    def format_text(self, state: dict[str, int]) -> str:
         """Converts the state of the subfolders to a string
 
         Parameters
         ==========
-        state: Dict[str, int]
+        state: dict[str, int]
             a dictionary mapping subfolder labels to new mail values
 
         Returns
@@ -112,9 +117,6 @@ class Maildir(base.ThreadPoolText):
         a string representation of the given state
         """
         if self.total:
-            return self._format_one(self.sub_folders[0]["label"],
-                                    sum(state.values()))
+            return self._format_one(self.sub_folders[0]["label"], sum(state.values()))
         else:
-            return self.separator.join(
-                self._format_one(*item) for item in state.items()
-            )
+            return self.separator.join(self._format_one(*item) for item in state.items())
