@@ -29,6 +29,7 @@
 from os import path
 
 from libqtile import bar, pangocffi, utils
+from libqtile.command.base import expose_command
 from libqtile.log_utils import logger
 from libqtile.notify import ClosedReason, notifier
 from libqtile.widget import base
@@ -50,7 +51,9 @@ class Notify(base._TextBox):
     defaults = [
         ("foreground_urgent", "ff0000", "Foreground urgent priority colour"),
         ("foreground_low", "dddddd", "Foreground low priority  colour"),
-        ("default_timeout", None, "Default timeout (seconds) for notifications"),
+        ("default_timeout_low", 5, "Default timeout (seconds) for low urgency notifications."),
+        ("default_timeout", 10, "Default timeout (seconds) for normal notifications"),
+        ("default_timeout_urgent", None, "Default timeout (seconds) for urgent notifications"),
         ("audiofile", None, "Audiofile played during notifications"),
         ("action", True, "Enable handling of default action upon right click"),
         (
@@ -62,6 +65,8 @@ class Notify(base._TextBox):
             "   return text.replace('\n', '')"
             "then set option parse_text=my_func",
         ),
+        ("background_urgent", "440000", "Background urgent priority colour"),
+        ("background_low", "444444", "Background low priority colour"),
     ]
     capabilities = {"body", "actions"}
 
@@ -76,11 +81,13 @@ class Notify(base._TextBox):
             "Button5": self.next,
         }
         if self.action:
-            default_callbacks["Button3"] = self.invoke
+            default_callbacks["Button3"] = self._invoke
         else:
             self.capabilities = Notify.capabilities.difference({"actions"})
         self.add_callbacks(default_callbacks)
 
+        self.background_normal = self.background
+
     def _configure(self, qtile, bar):
         base._TextBox._configure(self, qtile, bar)
         self.layout = self.drawer.textlayout(
@@ -89,6 +96,15 @@ class Notify(base._TextBox):
         if notifier is None:
             logger.warning("You must install dbus-next to use the Notify widget.")
 
+        # Create a tuple of our default timeouts. Urgency is an integer of 0-2
+        # (see https://specifications.freedesktop.org/notification-spec/notification-spec-latest.html#urgency-levels)
+        # so they will work as the index of the tuple.
+        self._timeouts = (
+            self.default_timeout_low,
+            self.default_timeout,
+            self.default_timeout_urgent,
+        )
+
     async def _config_async(self):
         if notifier is None:
             return
@@ -103,6 +119,10 @@ class Notify(base._TextBox):
                 utils.hex(self.foreground_urgent if urgency == 2 else self.foreground_low),
                 self.text,
             )
+            self.background = self.background_urgent if urgency == 2 else self.background_low
+        else:
+            self.background = self.background_normal
+
         if notif.body:
             self.text = '<span weight="bold">%s</span> - %s' % (
                 self.text,
@@ -114,7 +134,7 @@ class Notify(base._TextBox):
             except:  # noqa: E722
                 logger.exception("parse_text function failed:")
         if self.audiofile and path.exists(self.audiofile):
-            self.qtile.cmd_spawn("aplay -q '%s'" % self.audiofile)
+            self.qtile.spawn("aplay -q '%s'" % self.audiofile)
 
     def update(self, notif):
         self.qtile.call_soon_threadsafe(self.real_update, notif)
@@ -126,13 +146,22 @@ class Notify(base._TextBox):
             self.timeout_add(
                 notif.timeout / 1000, self.clear, method_args=(ClosedReason.expired,)
             )
-        elif self.default_timeout:
-            self.timeout_add(
-                self.default_timeout, self.clear, method_args=(ClosedReason.expired,)
-            )
+        else:
+            urgency = getattr(notif.hints.get("urgency"), "value", 1)
+            try:
+                timeout = self._timeouts[urgency]
+            except IndexError:
+                logger.warning(
+                    "Notification had an unexpected urgency value. Treating as normal priority."
+                )
+                timeout = self._timeouts[1]
+
+            if timeout:
+                self.timeout_add(timeout, self.clear, method_args=(ClosedReason.expired,))
         self.bar.draw()
         return True
 
+    @expose_command()
     def display(self):
         if notifier is None:
             return
@@ -140,12 +169,15 @@ class Notify(base._TextBox):
         self.set_notif_text(notifier.notifications[self.current_id])
         self.bar.draw()
 
+    @expose_command()
     def clear(self, reason=ClosedReason.dismissed):
+        """Clear the notification"""
         if notifier is None:
             return
 
         notifier._service.NotificationClosed(notifier.notifications[self.current_id].id, reason)
         self.text = ""
+        self.background = self.background_normal
         self.current_id = len(notifier.notifications) - 1
         self.bar.draw()
 
@@ -155,72 +187,44 @@ class Notify(base._TextBox):
             if notif.id == nid:
                 self.clear(ClosedReason.method)
 
+    @expose_command()
     def prev(self):
+        """Show previous notification."""
         if self.current_id > 0:
             self.current_id -= 1
         self.display()
 
+    @expose_command()
     def next(self):
         if notifier is None:
             return
 
+        """Show next notification."""
         if self.current_id < len(notifier.notifications) - 1:
             self.current_id += 1
             self.display()
 
-    def invoke(self):
+    def _invoke(self):
         if self.current_id < len(notifier.notifications):
             notif = notifier.notifications[self.current_id]
             if notif.actions:
                 notifier._service.ActionInvoked(notif.id, notif.actions[0])
             self.clear()
 
-    def cmd_display(self):
-        """Display the notifcication"""
-        self.display()
-
-    def cmd_clear(self):
-        """Clear the notification"""
-        self.clear()
-
-    def cmd_toggle(self):
+    @expose_command()
+    def toggle(self):
         """Toggle showing/clearing the notification"""
         if self.text == "":
             self.display()
         else:
             self.clear()
 
-    def cmd_prev(self):
-        """Show previous notification"""
-        self.prev()
-
-    def cmd_next(self):
-        """Show next notification"""
-        self.next()
-
-    def cmd_invoke(self):
+    @expose_command()
+    def invoke(self):
         """Invoke the notification's default action"""
         if self.action:
-            self.invoke()
+            self._invoke()
 
     def finalize(self):
-        # We may need some async calls as part of the finalize call
-        # We run this with `call_soon_threadsafe` as this waits for
-        # the job to finish before continuing. This is important as,
-        # if the config is just reloading, we need to finish deregistering
-        # the notification server before the new Notify widget instance
-        # registers and creates a new server.
-        self.qtile.call_soon_threadsafe(self._finalize)
-        base._TextBox.finalize(self)
-
-    async def _finalize(self):
-        if notifier is not None:
-            task = notifier.unregister(self.update)
-
-            # If the notifier has no more callbacks then it needs to be stopped.
-            # The returned task will handle the release of the service name from
-            # dbus. We await it here to make sure it's finished before we
-            # complete the finalisation of this widget.
-            if task:
-                await task
+        notifier.unregister(self.update, on_close=self.on_close)
         base._TextBox.finalize(self)
