@@ -17,8 +17,6 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
-import asyncio
 import os
 from functools import partial
 
@@ -42,7 +40,8 @@ except ImportError:
 from libqtile import bar
 from libqtile.images import Img
 from libqtile.log_utils import logger
-from libqtile.utils import add_signal_receiver
+from libqtile.resources.status_notifier.statusnotifieritem import STATUS_NOTIFIER_ITEM_SPEC
+from libqtile.utils import add_signal_receiver, create_task
 from libqtile.widget import base
 
 # StatusNotifier seems to have two potential interface names.
@@ -105,8 +104,12 @@ class StatusNotifierItem:  # noqa: E303
             try:
                 introspection = await self.bus.introspect(self.service, self.path)
                 found_path = True
+            except InvalidBusNameError:
+                # This is probably an Ayatana indicator which doesn't provide the service name.
+                # We'll pick it up via the message handler so we can ignore this.
+                return False
             except InvalidObjectPathError:
-                logger.info(f"Cannot find {self.path} path on {self.service}.")
+                logger.info("Cannot find %s path on %s.", self.path, self.service)
                 if self.path == STATUSNOTIFIER_PATH:
                     return False
 
@@ -132,11 +135,56 @@ class StatusNotifierItem:  # noqa: E303
                 continue
 
         if not interface_found:
-            logger.warning(f"Unable to find StatusNotifierItem" f"interface on {self.service}")
-            return False
+            logger.info(
+                "Unable to find StatusNotifierItem interface on %s. Falling back to default spec.",
+                self.service,
+            )
+            try:
+                obj = self.bus.get_proxy_object(
+                    self.service, STATUSNOTIFIER_PATH, STATUS_NOTIFIER_ITEM_SPEC
+                )
+                self.item = obj.get_interface("org.kde.StatusNotifierItem")
+            except InterfaceNotFoundError:
+                logger.warning(
+                    "Failed to find StatusNotifierItem interface on %s and fallback to default spec also failed.",
+                    self.service,
+                )
+                return False
 
-        # Default to XDG icon:
-        icon_name = await self.item.get_icon_name()
+        await self._get_local_icon()
+
+        # If there's no XDG icon, try to use icon provided by application
+        if self.icon:
+            self.item.on_new_icon(self._update_local_icon)
+
+        else:
+            # Get initial application icons:
+            for icon in ["Icon", "Attention", "Overlay"]:
+                await self._get_icon(icon)
+
+            if self.has_icons:
+                # Attach listeners for when the icon is updated
+                self.item.on_new_icon(self._new_icon)
+                self.item.on_new_attention_icon(self._new_attention_icon)
+                self.item.on_new_overlay_icon(self._new_overlay_icon)
+
+        if not self.has_icons:
+            logger.warning(
+                "Cannot find icon in current theme and no icon provided by StatusNotifierItem."
+            )
+
+        return True
+
+    async def _get_local_icon(self):
+        # Default to XDG icon
+        # Some implementations don't provide an IconName property so we
+        # need to catch an error if we can't read it.
+        # We can't use hasattr to check this as the method will be created
+        # where we've used the default XML spec to provide the object introspection
+        try:
+            icon_name = await self.item.get_icon_name()
+        except DBusError:
+            return
 
         try:
             icon_path = await self.item.get_icon_theme_path()
@@ -147,36 +195,22 @@ class StatusNotifierItem:  # noqa: E303
         if not self.icon:
             self.icon = self._get_xdg_icon(icon_name)
 
-        # If there's no XDG icon, try to use icon provided by application
-        if self.icon is None:
+    def _create_task_and_draw(self, coro):
+        task = create_task(coro)
+        task.add_done_callback(self._redraw)
 
-            # Get initial application icons:
-            for icon in ["Icon", "Attention", "Overlay"]:
-                await self._get_icon(icon)
-
-            # Attach listeners for when the icon is updated
-            self.item.on_new_icon(self._new_icon)
-            self.item.on_new_attention_icon(self._new_attention_icon)
-            self.item.on_new_overlay_icon(self._new_overlay_icon)
-
-        if not self.has_icons:
-            logger.warning(
-                "Cannot find icon in current theme and " "no icon provided by StatusNotifierItem."
-            )
-
-        return True
+    def _update_local_icon(self):
+        self.icon = None
+        self._create_task_and_draw(self._get_local_icon())
 
     def _new_icon(self):
-        task = asyncio.create_task(self._get_icon("Icon"))
-        task.add_done_callback(self._redraw)
+        self._create_task_and_draw(self._get_icon("Icon"))
 
     def _new_attention_icon(self):
-        task = asyncio.create_task(self._get_icon("Attention"))
-        task.add_done_callback(self._redraw)
+        self._create_task_and_draw(self._get_icon("Attention"))
 
     def _new_overlay_icon(self):
-        task = asyncio.create_task(self._get_icon("Overlay"))
-        task.add_done_callback(self._redraw)
+        self._create_task_and_draw(self._get_icon("Overlay"))
 
     def _get_custom_icon(self, icon_name, icon_path):
         for ext in [".png", ".svg"]:
@@ -203,7 +237,9 @@ class StatusNotifierItem:  # noqa: E303
         adds to an internal dictionary for later retrieval.
         """
         attr, method = self.icon_map[icon_name]
-        pixmap = getattr(self.item, method)
+        pixmap = getattr(self.item, method, None)
+        if pixmap is None:
+            return
         icon_pixmap = await pixmap()
 
         # Items can present multiple pixmaps for different
@@ -231,7 +267,7 @@ class StatusNotifierItem:  # noqa: E303
         """Method to invalidate icon cache and redraw icons."""
         self._invalidate_icons()
         if self.on_icon_changed is not None:
-            self.on_icon_changed()
+            self.on_icon_changed(self)
 
     def _invalidate_icons(self):
         self.surfaces = {}
@@ -319,7 +355,8 @@ class StatusNotifierItem:  # noqa: E303
         return icon
 
     def activate(self):
-        asyncio.create_task(self._activate())
+        if hasattr(self.item, "call_activate"):
+            create_task(self._activate())
 
     async def _activate(self):
         # Call Activate method and pass window position hints
@@ -373,9 +410,9 @@ class StatusNotifierWatcher(ServiceInterface):  # noqa: E303
         if message.member != "RegisterStatusNotifierItem":
             return False
 
-        # If the argument passed to the method is the service name then we
-        # don't need to do anything else.
-        if message.sender == message.body[0]:
+        # If the argument is not an object path (starting with "/") then we assume
+        # it is the bus name and we don't need to do anything else.
+        if not message.body[0].startswith("/"):
             return False
 
         if message.sender not in self._items:
@@ -477,6 +514,10 @@ class StatusNotifierHost:  # noqa: E303
         self.items: List[StatusNotifierItem] = []
         self.name = "qtile"
         self.icon_theme: str = None
+        self.started = False
+        self._on_item_added: List[Callable] = []
+        self._on_item_removed: List[Callable] = []
+        self._on_icon_changed: List[Callable] = []
 
     async def start(
         self,
@@ -484,10 +525,28 @@ class StatusNotifierHost:  # noqa: E303
         on_item_removed: Optional[Callable] = None,
         on_icon_changed: Optional[Callable] = None,
     ):
+        """
+        Starts the host if not already started.
+
+        Widgets should register their callbacks via this method.
+        """
+        if on_item_added:
+            self._on_item_added.append(on_item_added)
+
+        if on_item_removed:
+            self._on_item_removed.append(on_item_removed)
+
+        if on_icon_changed:
+            self._on_icon_changed.append(on_icon_changed)
+
+        if self.started:
+            if on_item_added:
+                for item in self.items:
+                    on_item_added(item)
+            return
+
         self.bus = await MessageBus().connect()
-        self.on_item_added = on_item_added
-        self.on_item_removed = on_item_removed
-        self.on_icon_changed = on_icon_changed
+        await self.bus.request_name("org.freedesktop.StatusNotifierHost-qtile")
         for iface in BUS_NAMES:
             w = StatusNotifierWatcher(iface)
             w.on_item_added = self.add_item
@@ -498,6 +557,7 @@ class StatusNotifierHost:  # noqa: E303
             # the host on the bus.
             w.RegisterStatusNotifierHost(self.name)
             self.watchers.append(w)
+            self.started = True
 
     def item_added(self, item, service, future):
         success = future.result()
@@ -505,8 +565,16 @@ class StatusNotifierHost:  # noqa: E303
         # add to our list and redraw the bar
         if success:
             self.items.append(item)
-            if self.on_item_added:
-                self.on_item_added(item)
+            for callback in self._on_item_added:
+                callback(item)
+
+        # It's an invalid item so let's remove it from the watchers
+        else:
+            for w in self.watchers:
+                try:
+                    w._items.remove(service)
+                except ValueError:
+                    pass
 
     def add_item(self, service, path=None):
         """
@@ -514,9 +582,9 @@ class StatusNotifierHost:  # noqa: E303
         start it.
         """
         item = StatusNotifierItem(self.bus, service, path=path, icon_theme=self.icon_theme)
-        item.on_icon_changed = self.on_icon_changed
+        item.on_icon_changed = self.item_icon_changed
         if item not in self.items:
-            task = asyncio.create_task(item.start())
+            task = create_task(item.start())
             task.add_done_callback(partial(self.item_added, item, service))
 
     def remove_item(self, interface):
@@ -524,8 +592,12 @@ class StatusNotifierHost:  # noqa: E303
         # remove it and redraw the bar
         if interface in self.items:
             self.items.remove(interface)
-            if self.on_item_removed:
-                self.on_item_removed(interface)
+            for callback in self._on_item_removed:
+                callback(interface)
+
+    def item_icon_changed(self, item):
+        for callback in self._on_icon_changed:
+            callback(item)
 
 
 host = StatusNotifierHost()  # noqa: E303
@@ -539,10 +611,10 @@ class StatusNotifier(base._Widget):
     As per the specification, app icons are first retrieved from the
     user's current theme. If this is not available then the app may
     provide its own icon. In order to use this functionality, users
-    are recommended to install the `xdg`_ module to support retrieving
-    icons from the selected theme.
+    are recommended to install the `pyxdg <https://pypi.org/project/pyxdg/>`__
+    module to support retrieving icons from the selected theme.
 
-    Letf-clicking an icon will trigger an activate event.
+    Left-clicking an icon will trigger an activate event.
 
     .. note::
 
@@ -550,8 +622,6 @@ class StatusNotifier(base._Widget):
         However, a modded version of the widget which provides basic menu
         support is available from elParaguayo's `qtile-extras
         <https://github.com/elParaguayo/qtile-extras>`_ repo.
-
-    .. _xdg: https://pypi.org/project/xdg/
     """
 
     orientations = base.ORIENTATION_BOTH
